package cn.wisenergy.chnmuseum.party.service.impl;

import com.baidu.ueditor.define.AppInfo;
import com.baidu.ueditor.define.BaseState;
import com.baidu.ueditor.define.MultiState;
import com.baidu.ueditor.define.State;
import com.baidu.ueditor.extend.UeditorService;
import cn.wisenergy.chnmuseum.party.common.dfs.FastdfsUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Map;

/**
 * UeditorService实现 - Fastdfs
 */
@Service
@ConfigurationProperties
public class UeditorServiceFastdfsImpl implements UeditorService {

	@Value("${dfsFileAccessBasePath}")
	private String dfsFileAccessBasePath;

	@Override
	public com.baidu.ueditor.extend.MultipartFile getMultipartFile(String filedName, HttpServletRequest request) {
		com.baidu.ueditor.extend.MultipartFile resultFile = null;
		try {
			MultipartHttpServletRequest multipartHttpservletRequest = (MultipartHttpServletRequest) request;
			MultipartFile multipartFile = multipartHttpservletRequest.getFile(filedName);
			if (!multipartFile.isEmpty()) {
				resultFile = new com.baidu.ueditor.extend.StandardMultipartFile(filedName, multipartFile.getInputStream(),
						multipartFile.getOriginalFilename(), multipartFile.getSize());
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return resultFile;
	}

	@Override
	public State saveFileByInputStream(com.baidu.ueditor.extend.MultipartFile multipartFile, long maxSize) {
		State state = null;
		try {
			if (multipartFile.getSize() > maxSize) {
				return new BaseState(false, AppInfo.MAX_SIZE);
			}

			Map<String, Object> uploadResult = null;
			String originalFileName = multipartFile.getOriginalFilename();
			uploadResult = FastdfsUtil.uploadFile(multipartFile.getBytes(), originalFileName);

//			// 文件新路径
//			uploadResult = FastdfsUtil.uploadPic(multipartFile.getBytes(), multipartFile.getOriginalFilename(),
//					multipartFile.getSize());


			if ((Boolean) uploadResult.get("status")) {
				state = new BaseState(true);
				state.putInfo("size", uploadResult.get("length").toString());
				state.putInfo("title", uploadResult.get("fileName").toString());
				// state.putInfo("groupName",
				// uploadResult.get("groupName").toString());
				// state.putInfo("storageFileName",
				// uploadResult.get("storageFileName").toString());
				state.putInfo("url", dfsFileAccessBasePath + "/" + uploadResult.get("link").toString());

				// 把上传的文件信息记入数据库
				// ---自行处理---
				return state;
			}
		} catch (IOException e) {

		}
		return new BaseState(false, AppInfo.IO_ERROR);
	}

	@Override
	public State saveBinaryFile(byte[] data, String fileName) {
		State state = null;

		Map<String, Object> uploadResult = FastdfsUtil.uploadFile(data, fileName);
		if ((Boolean) uploadResult.get("status")) {
			state = new BaseState(true);
			state.putInfo("size", uploadResult.get("length").toString());
			state.putInfo("title", uploadResult.get("fileName").toString());
			// state.putInfo("groupName",
			// uploadResult.get("groupName").toString());
			// state.putInfo("storageFileName",
			// uploadResult.get("storageFileName").toString());
			state.putInfo("url", dfsFileAccessBasePath + "/" + uploadResult.get("link").toString());

			// 把上传的文件信息记入数据库
			// ---自行处理---
			return state;
		}

		return new BaseState(false, AppInfo.IO_ERROR);
	}

	@Override
	public State listFile(String[] allowFiles, int start, int pageSize) {
		// 把计入数据库中的文件信息读取出来，返回即可

		// 下面的代码，仅作示例
		State state = new MultiState(true);
		state.putInfo("start", start);
		state.putInfo("total", 0);
		return state;
	}
}
