package cn.wisenergy.chnmuseum.party.web.controller.base;

import cn.wisenergy.chnmuseum.party.auth.util.JwtTokenUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SimplePropertyPreFilter;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.Serializable;
import java.util.List;

/**
 * controller 基类控制器
 */
public class BaseController implements Serializable {

    private static final long serialVersionUID = 624841049563451448L;
    protected static final Logger logger = LoggerFactory.getLogger(BaseController.class);

    @Resource
    protected HttpServletRequest request;

    /**
     * <p>
     * 获取分页对象
     * </p>
     */
    protected <T> Page<T> getPage() {
        return getPage(10L);
    }

    protected <T> Page<T> getPage(long size) {
        long _size = size, _index = 1;
        if (request.getParameter("_size") != null) {
            _size = Integer.parseInt(request.getParameter("_size"));
        }
        if (request.getParameter("_index") != null) {
            _index = Integer.parseInt(request.getParameter("_index"));
        }
        return new Page<>(_index, _size);
    }

    /**
     * JSON 过滤相关字段
     *
     * @param obj          待过滤的对象
     * @param propertyList 需要过滤掉的字段集合
     * @return 过滤后的JSON结果
     */
    protected String propertyFilter(Object obj, List<String> propertyList) {
        SimplePropertyPreFilter filter = new SimplePropertyPreFilter();
        for (String params : propertyList) {
            filter.getExcludes().add(params);
        }
        return JSONObject.toJSONString(obj, filter);
    }

    /**
     * JSON 过滤相关字段，并保持结果为JSON对象
     *
     * @param obj          待过滤的对象
     * @param propertyList 需要过滤掉的字段集合
     * @return 过滤后的JSON对象结果
     */
    protected JSONObject propertyFilterToJsonObject(Object obj, List<String> propertyList) {
        SimplePropertyPreFilter filter = new SimplePropertyPreFilter();
        for (String params : propertyList) {
            filter.getExcludes().add(params);
        }
        return JSONObject.parseObject(JSONObject.toJSONString(obj, filter));
    }

    /**
     * JSON 过滤相关字段，并保持结果为JSON数组
     *
     * @param obj          待过滤的对象
     * @param propertyList 需要过滤掉的字段集合
     * @return 过滤后的JSON数组结果
     */
    protected JSONArray propertyFilterToJsonArray(Object obj, List<String> propertyList) {
        SimplePropertyPreFilter filter = new SimplePropertyPreFilter();
        for (String params : propertyList) {
            filter.getExcludes().add(params);
        }
        return JSONObject.parseArray(JSONObject.toJSONString(obj, filter));
    }

    /**
     * <p>
     * 根据token信息
     * </p>
     */
    protected String getUserId() {
        String authorization = request.getHeader("Authorization");
        if (StringUtils.isNotBlank(authorization)) {
            if (authorization.startsWith("Bearer ")) {
                authorization = authorization.substring(7);
            }
        }
        return JwtTokenUtil.getEmployeeId(authorization);
    }

    /**
     * <p>
     * 根据token信息
     * </p>
     */
    protected String getUserName() {
        String authorization = request.getHeader("Authorization");
        if (StringUtils.isNotBlank(authorization)) {
            if (authorization.startsWith("Bearer ")) {
                authorization = authorization.substring(7);
            }
        }
        return JwtTokenUtil.getUsername(authorization);
    }

}
